=== modified file 'Makefile'
--- Makefile	2009-05-05 07:17:32 +0000
+++ Makefile	2009-05-28 18:03:38 +0000
@@ -221,6 +221,7 @@
 # make bzr.exe for win32 with py2exe
 exe:
 	@echo *** Make bzr.exe
+	$(PYTHON) tools/win32/ostools.py remove bzrlib/*.pyd
 	$(PYTHON) setup.py build_ext -i -f $(PYTHON_BUILDFLAGS)
 	$(PYTHON) setup.py py2exe > py2exe.log
 	$(PYTHON) tools/win32/ostools.py copytodir tools/win32/start_bzr.bat win32_bzr.exe

=== modified file 'NEWS'
--- NEWS	2009-05-29 11:26:30 +0000
+++ NEWS	2009-06-02 13:48:27 +0000
@@ -12,12 +12,22 @@
 Compatibility Breaks
 ********************
 
+* Display prompt on stderr (instead of stdout) when querying users so
+  that the output of commands can be safely redirected.
+  (Vincent Ladeuil, #376582)
+
+
 New Features
 ************
 
 Improvements
 ************
 
+
+* ``--development6-rich-root`` can now stack. (Modulo some smart-server
+  bugs with stacking and non default formats.)
+  (John Arbash Meinel, #373455)
+
 * Numerous operations are now faster for huge projects, i.e. those
   with a large number of files and/or a large number of revisions,
   particularly when the latest development format is used. These
@@ -30,10 +40,6 @@
 
   (Ian Clatworthy)
 
-* ``--development6-rich-root`` can now stack. (Modulo some smart-server
-  bugs with stacking and non default formats.)
-  (John Arbash Meinel, #373455)
-
 
 Bug Fixes
 *********
@@ -45,6 +51,11 @@
   ``RemoteRepository`` was handling fallbacks along with the
   ``_real_repository``. (Andrew Bennetts, John Arbash Meinel, #375496)
 
+* Fix a small bug with fetching revisions w/ ghosts into a new stacked
+  branch. Not often triggered, because it required ghosts to be part of
+  the fetched revisions, not in the stacked-on ancestry.
+  (John Arbash Meinel)
+
 
 Documentation
 *************

=== modified file 'bzrlib/foreign.py'
--- bzrlib/foreign.py	2009-05-23 04:55:52 +0000
+++ bzrlib/foreign.py	2009-06-01 02:55:34 +0000
@@ -119,25 +119,6 @@
         self.mapping = mapping
 
 
-def show_foreign_properties(rev):
-    """Custom log displayer for foreign revision identifiers.
-
-    :param rev: Revision object.
-    """
-    # Revision comes directly from a foreign repository
-    if isinstance(rev, ForeignRevision):
-        return rev.mapping.vcs.show_foreign_revid(rev.foreign_revid)
-
-    # Revision was once imported from a foreign repository
-    try:
-        foreign_revid, mapping = \
-            foreign_vcs_registry.parse_revision_id(rev.revision_id)
-    except errors.InvalidRevisionId:
-        return {}
-
-    return mapping.vcs.show_foreign_revid(foreign_revid)
-
-
 class ForeignVcs(object):
     """A foreign version control system."""
 
@@ -179,7 +160,7 @@
         :param revid: The bzr revision id
         :return: tuple with foreign revid and vcs mapping
         """
-        if not "-" in revid:
+        if not ":" in revid or not "-" in revid:
             raise errors.InvalidRevisionId(revid, None)
         try:
             foreign_vcs = self.get(revid.split("-")[0])

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2009-04-20 08:37:32 +0000
+++ bzrlib/log.py	2009-06-01 02:55:34 +0000
@@ -69,6 +69,7 @@
     config,
     diff,
     errors,
+    foreign,
     repository as _mod_repository,
     revision as _mod_revision,
     revisionspec,
@@ -1366,14 +1367,42 @@
         else:
             return ''
 
+    def show_foreign_info(self, rev, indent):
+        """Custom log displayer for foreign revision identifiers.
+
+        :param rev: Revision object.
+        """
+        # Revision comes directly from a foreign repository
+        if isinstance(rev, foreign.ForeignRevision):
+            self._write_properties(indent, rev.mapping.vcs.show_foreign_revid(
+                rev.foreign_revid))
+            return
+
+        # Imported foreign revision revision ids always contain :
+        if not ":" in rev.revision_id:
+            return
+
+        # Revision was once imported from a foreign repository
+        try:
+            foreign_revid, mapping = \
+                foreign.foreign_vcs_registry.parse_revision_id(rev.revision_id)
+        except errors.InvalidRevisionId:
+            return
+
+        self._write_properties(indent, 
+            mapping.vcs.show_foreign_revid(foreign_revid))
+
     def show_properties(self, revision, indent):
         """Displays the custom properties returned by each registered handler.
 
         If a registered handler raises an error it is propagated.
         """
         for key, handler in properties_handler_registry.iteritems():
-            for key, value in handler(revision).items():
-                self.to_file.write(indent + key + ': ' + value + '\n')
+            self._write_properties(indent, handler(revision))
+
+    def _write_properties(self, indent, properties):
+        for key, value in properties.items():
+            self.to_file.write(indent + key + ': ' + value + '\n')
 
     def show_diff(self, to_file, diff, indent):
         for l in diff.rstrip().split('\n'):
@@ -1403,6 +1432,7 @@
             to_file.write('\n')
             for parent_id in revision.rev.parent_ids:
                 to_file.write(indent + 'parent: %s\n' % (parent_id,))
+        self.show_foreign_info(revision.rev, indent)
         self.show_properties(revision.rev, indent)
 
         committer = revision.rev.committer
@@ -1485,6 +1515,7 @@
                             self.show_timezone, date_fmt="%Y-%m-%d",
                             show_offset=False),
                 tags, self.merge_marker(revision)))
+        self.show_foreign_info(revision.rev, indent+offset)
         self.show_properties(revision.rev, indent+offset)
         if self.show_ids:
             to_file.write(indent + offset + 'revision-id:%s\n'
@@ -1901,9 +1932,6 @@
 
 
 properties_handler_registry = registry.Registry()
-properties_handler_registry.register_lazy("foreign",
-                                          "bzrlib.foreign",
-                                          "show_foreign_properties")
 
 
 # adapters which revision ids to log are filtered. When log is called, the

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2009-06-02 13:48:27 +0000
@@ -335,12 +335,14 @@
         g_conf = config.GlobalConfig()
         g_conf.set_user_option('email', 'Test User <test@user.com>')
         stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
         ui.ui_factory = tests.TestUIFactory(stdin='userpass\n',
-                                            stdout=stdout)
+                                            stdout=stdout, stderr=stderr)
         self.assertIs(None, service.registrant_password)
         service.gather_user_credentials()
         self.assertEqual('test@user.com', service.registrant_email)
         self.assertEqual('userpass', service.registrant_password)
-        self.assertContainsRe(stdout.getvalue(),
+        self.assertEquals('', stdout.getvalue())
+        self.assertContainsRe(stderr.getvalue(),
                              'launchpad.net password for test@user\\.com')
 

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2009-05-29 10:25:37 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2009-06-01 18:13:46 +0000
@@ -857,7 +857,7 @@
         self._chk_id_roots = None
         self._chk_p_id_roots = None
 
-    def _get_inventory_stream(self, inventory_keys):
+    def _get_inventory_stream(self, inventory_keys, allow_absent=False):
         """Get a stream of inventory texts.
 
         When this function returns, self._chk_id_roots and self._chk_p_id_roots
@@ -872,6 +872,11 @@
             stream = source_vf.get_record_stream(inventory_keys,
                                                  'groupcompress', True)
             for record in stream:
+                if record.storage_kind == 'absent':
+                    if allow_absent:
+                        continue
+                    else:
+                        raise errors.NoSuchRevision(self, record.key)
                 bytes = record.get_bytes_as('fulltext')
                 chk_inv = inventory.CHKInventory.deserialise(None, bytes,
                                                              record.key)
@@ -981,7 +986,11 @@
             raise AssertionError('Cannot call get_stream_for_missing_keys'
                 ' untill all of get_stream() has been consumed.')
         # Yield the inventory stream, so we can find the chk stream
-        yield self._get_inventory_stream(missing_inventory_keys)
+        # Some of the missing_keys will be missing because they are ghosts.
+        # As such, we can ignore them. The Sink is required to verify there are
+        # no unavailable texts when the ghost inventories are not filled in.
+        yield self._get_inventory_stream(missing_inventory_keys,
+                                         allow_absent=True)
         # We use the empty set for excluded_revision_ids, to make it clear that
         # we want to transmit all referenced chk pages.
         for stream_info in self._get_filtered_chk_streams(set()):

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-05-27 15:39:12 +0000
+++ bzrlib/repository.py	2009-06-01 18:13:46 +0000
@@ -4265,8 +4265,12 @@
                 continue
             # Ask for full texts always so that we don't need more round trips
             # after this stream.
-            stream = vf.get_record_stream(keys,
-                self.to_format._fetch_order, True)
+            # Some of the missing keys are genuinely ghosts, so filter absent
+            # records. The Sink is responsible for doing another check to
+            # ensure that ghosts don't introduce missing data for future
+            # fetches.
+            stream = versionedfile.filter_absent(vf.get_record_stream(keys,
+                self.to_format._fetch_order, True))
             yield substream_kind, stream
 
     def inventory_fetch_order(self):

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2009-05-07 05:08:46 +0000
+++ bzrlib/smart/repository.py	2009-06-01 18:13:46 +0000
@@ -436,6 +436,8 @@
         for record in substream:
             if record.storage_kind in ('chunked', 'fulltext'):
                 serialised = record_to_fulltext_bytes(record)
+            elif record.storage_kind == 'absent':
+                raise ValueError("Absent factory for %s" % (record.key,))
             else:
                 serialised = record.get_bytes_as(record.storage_kind)
             if serialised:

=== modified file 'bzrlib/tests/per_repository/test_fetch.py'
--- bzrlib/tests/per_repository/test_fetch.py	2009-05-23 04:55:52 +0000
+++ bzrlib/tests/per_repository/test_fetch.py	2009-06-01 18:13:46 +0000
@@ -23,6 +23,7 @@
     graph,
     remote,
     repository,
+    tests,
     )
 from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import TestSkipped
@@ -145,7 +146,7 @@
             ])
 
     def test_fetch_to_rich_root_set_parent_1_parent(self):
-        # 1 parent rev -> 1 parent 
+        # 1 parent rev -> 1 parent
         self.do_test_fetch_to_rich_root_sets_parents_correctly(
             ((ROOT_ID, 'base'),),
             [('base', None, [('add', ('', ROOT_ID, 'directory', ''))]),
@@ -300,6 +301,55 @@
         repo.fetch(tree.branch.repository)
         repo.fetch(tree.branch.repository)
 
+    def make_simple_branch_with_ghost(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'content\n'))])
+        builder.build_snapshot('B-id', ['A-id', 'ghost-id'], [])
+        builder.finish_series()
+        source_b = builder.get_branch()
+        source_b.lock_read()
+        self.addCleanup(source_b.unlock)
+        return source_b
+
+    def test_fetch_with_ghost(self):
+        source_b = self.make_simple_branch_with_ghost()
+        target = self.make_repository('target')
+        target.lock_write()
+        self.addCleanup(target.unlock)
+        target.fetch(source_b.repository, revision_id='B-id')
+
+    def test_fetch_into_smart_with_ghost(self):
+        trans = self.make_smart_server('target')
+        source_b = self.make_simple_branch_with_ghost()
+        target = self.make_repository('target')
+        # Re-open the repository over the smart protocol
+        target = repository.Repository.open(trans.base)
+        target.lock_write()
+        self.addCleanup(target.unlock)
+        try:
+            target.fetch(source_b.repository, revision_id='B-id')
+        except errors.TokenLockingNotSupported:
+            # The code inside fetch() that tries to lock and then fails, also
+            # causes weird problems with 'lock_not_held' later on...
+            target.lock_read()
+            raise tests.KnownFailure('some repositories fail to fetch'
+                ' via the smart server because of locking issues.')
+
+    def test_fetch_from_smart_with_ghost(self):
+        trans = self.make_smart_server('source')
+        source_b = self.make_simple_branch_with_ghost()
+        target = self.make_repository('target')
+        target.lock_write()
+        self.addCleanup(target.unlock)
+        # Re-open the repository over the smart protocol
+        source = repository.Repository.open(trans.base)
+        source.lock_read()
+        self.addCleanup(source.unlock)
+        target.fetch(source, revision_id='B-id')
+
 
 class TestSource(TestCaseWithRepository):
     """Tests for/about the results of Repository._get_source."""

=== modified file 'bzrlib/tests/per_repository_reference/test_fetch.py'
--- bzrlib/tests/per_repository_reference/test_fetch.py	2009-05-29 09:56:12 +0000
+++ bzrlib/tests/per_repository_reference/test_fetch.py	2009-06-01 18:13:46 +0000
@@ -15,7 +15,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from bzrlib.smart import server
+from bzrlib import (
+    branch,
+    errors,
+    )
+from bzrlib.smart import (
+    server,
+    )
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
@@ -99,3 +105,47 @@
         final2_b = target_b.bzrdir.sprout('final2',
                                           revision_id='C-id').open_branch()
         self.assertEqual('C-id', final_b.last_revision())
+
+    def make_source_with_ghost_and_stacked_target(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'content\n'))])
+        builder.build_snapshot('B-id', ['A-id', 'ghost-id'], [])
+        builder.finish_series()
+        source_b = builder.get_branch()
+        source_b.lock_read()
+        self.addCleanup(source_b.unlock)
+        base = self.make_branch('base')
+        base.pull(source_b, stop_revision='A-id')
+        stacked = self.make_branch('stacked')
+        stacked.set_stacked_on_url('../base')
+        return source_b, base, stacked
+
+    def test_fetch_with_ghost_stacked(self):
+        (source_b, base,
+         stacked) = self.make_source_with_ghost_and_stacked_target()
+        stacked.pull(source_b, stop_revision='B-id')
+
+    def test_fetch_into_smart_stacked_with_ghost(self):
+        (source_b, base,
+         stacked) = self.make_source_with_ghost_and_stacked_target()
+        # Now, create a smart server on 'stacked' and re-open to force the
+        # target to be a smart target
+        trans = self.make_smart_server('stacked')
+        stacked = branch.Branch.open(trans.base)
+        stacked.lock_write()
+        self.addCleanup(stacked.unlock)
+        stacked.pull(source_b, stop_revision='B-id')
+
+    def test_fetch_to_stacked_from_smart_with_ghost(self):
+        (source_b, base,
+         stacked) = self.make_source_with_ghost_and_stacked_target()
+        # Now, create a smart server on 'source' and re-open to force the
+        # target to be a smart target
+        trans = self.make_smart_server('source')
+        source_b = branch.Branch.open(trans.base)
+        source_b.lock_read()
+        self.addCleanup(source_b.unlock)
+        stacked.pull(source_b, stop_revision='B-id')

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2009-04-27 16:10:10 +0000
+++ bzrlib/tests/test_config.py	2009-05-15 10:14:51 +0000
@@ -1461,7 +1461,8 @@
     """Test AuthenticationConfig behaviour"""
 
     def _check_default_password_prompt(self, expected_prompt_format, scheme,
-                              host=None, port=None, realm=None, path=None):
+                                       host=None, port=None, realm=None,
+                                       path=None):
         if host is None:
             host = 'bar.org'
         user, password = 'jim', 'precious'
@@ -1470,17 +1471,20 @@
             'user': user, 'realm': realm}
 
         stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
         ui.ui_factory = tests.TestUIFactory(stdin=password + '\n',
-                                            stdout=stdout)
+                                            stdout=stdout, stderr=stderr)
         # We use an empty conf so that the user is always prompted
         conf = config.AuthenticationConfig()
         self.assertEquals(password,
                           conf.get_password(scheme, host, user, port=port,
                                             realm=realm, path=path))
-        self.assertEquals(stdout.getvalue(), expected_prompt)
+        self.assertEquals(expected_prompt, stderr.getvalue())
+        self.assertEquals('', stdout.getvalue())
 
     def _check_default_username_prompt(self, expected_prompt_format, scheme,
-                              host=None, port=None, realm=None, path=None):
+                                       host=None, port=None, realm=None,
+                                       path=None):
         if host is None:
             host = 'bar.org'
         username = 'jim'
@@ -1488,13 +1492,15 @@
             'scheme': scheme, 'host': host, 'port': port,
             'realm': realm}
         stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
         ui.ui_factory = tests.TestUIFactory(stdin=username+ '\n',
-                                            stdout=stdout)
+                                            stdout=stdout, stderr=stderr)
         # We use an empty conf so that the user is always prompted
         conf = config.AuthenticationConfig()
         self.assertEquals(username, conf.get_user(scheme, host, port=port,
                           realm=realm, path=path, ask=True))
-        self.assertEquals(stdout.getvalue(), expected_prompt)
+        self.assertEquals(expected_prompt, stderr.getvalue())
+        self.assertEquals('', stdout.getvalue())
 
     def test_username_defaults_prompts(self):
         # HTTP prompts can't be tested here, see test_http.py

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2009-05-23 04:55:52 +0000
+++ bzrlib/tests/test_foreign.py	2009-06-01 02:55:34 +0000
@@ -301,36 +301,6 @@
         self.assertEquals(mapp, rev.mapping)
 
 
-class ShowForeignPropertiesTests(TestCase):
-    """Tests for the show_foreign_properties() function."""
-
-    def setUp(self):
-        super(ShowForeignPropertiesTests, self).setUp()
-        self.vcs = DummyForeignVcs()
-        foreign.foreign_vcs_registry.register("dummy",
-            self.vcs, "Dummy VCS")
-
-    def tearDown(self):
-        super(ShowForeignPropertiesTests, self).tearDown()
-        foreign.foreign_vcs_registry.remove("dummy")
-
-    def test_show_non_foreign(self):
-        """Test use with a native (non-foreign) bzr revision."""
-        self.assertEquals({}, foreign.show_foreign_properties(Revision("arevid")))
-
-    def test_show_imported(self):
-        rev = Revision("dummy-v1:my-foreign-revid")
-        self.assertEquals({ "dummy ding": "my/foreign\\revid" },
-                          foreign.show_foreign_properties(rev))
-
-    def test_show_direct(self):
-        rev = foreign.ForeignRevision(("some", "foreign", "revid"),
-                                      DummyForeignVcsMapping(self.vcs),
-                                      "roundtrip-revid")
-        self.assertEquals({ "dummy ding": "some/foreign\\revid" },
-                          foreign.show_foreign_properties(rev))
-
-
 class WorkingTreeFileUpdateTests(TestCaseWithTransport):
     """Tests for update_workingtree_fileids()."""
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2009-05-27 13:59:47 +0000
+++ bzrlib/tests/test_http.py	2009-06-02 09:21:20 +0000
@@ -1569,15 +1569,18 @@
         self.server.add_user('joe', 'foo')
         t = self.get_user_transport(None, None)
         stdout = tests.StringIOWrapper()
-        ui.ui_factory = tests.TestUIFactory(stdin='joe\nfoo\n', stdout=stdout)
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdin='joe\nfoo\n',
+                                            stdout=stdout, stderr=stderr)
         self.assertEqual('contents of a\n',t.get('a').read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
-        stdout.seek(0)
+        stderr.seek(0)
         expected_prompt = self._expected_username_prompt(t._unqualified_scheme)
-        self.assertEquals(expected_prompt, stdout.read(len(expected_prompt)))
+        self.assertEquals(expected_prompt, stderr.read(len(expected_prompt)))
+        self.assertEquals('', stdout.getvalue())
         self._check_password_prompt(t._unqualified_scheme, 'joe',
-                                    stdout.readline())
+                                    stderr.readline())
 
     def test_prompt_for_password(self):
         if self._testing_pycurl():
@@ -1588,12 +1591,15 @@
         self.server.add_user('joe', 'foo')
         t = self.get_user_transport('joe', None)
         stdout = tests.StringIOWrapper()
-        ui.ui_factory = tests.TestUIFactory(stdin='foo\n', stdout=stdout)
-        self.assertEqual('contents of a\n',t.get('a').read())
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdin='foo\n',
+                                            stdout=stdout, stderr=stderr)
+        self.assertEqual('contents of a\n', t.get('a').read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
         self._check_password_prompt(t._unqualified_scheme, 'joe',
-                                    stdout.getvalue())
+                                    stderr.getvalue())
+        self.assertEquals('', stdout.getvalue())
         # And we shouldn't prompt again for a different request
         # against the same transport.
         self.assertEqual('contents of b\n',t.get('b').read())

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2009-04-24 13:30:48 +0000
+++ bzrlib/tests/test_ui.py	2009-05-15 10:14:51 +0000
@@ -67,15 +67,17 @@
         self.assertEqual('', stdout.getvalue())
 
     def test_text_factory_ascii_password(self):
-        ui = TestUIFactory(stdin='secret\n', stdout=StringIOWrapper())
+        ui = TestUIFactory(stdin='secret\n', stdout=StringIOWrapper(),
+                           stderr=StringIOWrapper())
         pb = ui.nested_progress_bar()
         try:
             self.assertEqual('secret',
                              self.apply_redirected(ui.stdin, ui.stdout,
-                                                   ui.stdout,
+                                                   ui.stderr,
                                                    ui.get_password))
             # ': ' is appended to prompt
-            self.assertEqual(': ', ui.stdout.getvalue())
+            self.assertEqual(': ', ui.stderr.getvalue())
+            self.assertEqual('', ui.stdout.readline())
             # stdin should be empty
             self.assertEqual('', ui.stdin.readline())
         finally:
@@ -88,21 +90,22 @@
         it to utf8 to test that we transport the password correctly.
         """
         ui = TestUIFactory(stdin=u'baz\u1234'.encode('utf8'),
-                           stdout=StringIOWrapper())
-        ui.stdin.encoding = 'utf8'
-        ui.stdout.encoding = ui.stdin.encoding
+                           stdout=StringIOWrapper(),
+                           stderr=StringIOWrapper())
+        ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = 'utf8'
         pb = ui.nested_progress_bar()
         try:
-            password = self.apply_redirected(ui.stdin, ui.stdout, ui.stdout,
+            password = self.apply_redirected(ui.stdin, ui.stdout, ui.stderr,
                                              ui.get_password,
                                              u'Hello \u1234 %(user)s',
                                              user=u'some\u1234')
             # We use StringIO objects, we need to decode them
             self.assertEqual(u'baz\u1234', password.decode('utf8'))
             self.assertEqual(u'Hello \u1234 some\u1234: ',
-                             ui.stdout.getvalue().decode('utf8'))
-            # stdin should be empty
+                             ui.stderr.getvalue().decode('utf8'))
+            # stdin and stdout should be empty
             self.assertEqual('', ui.stdin.readline())
+            self.assertEqual('', ui.stdout.readline())
         finally:
             pb.finished()
 
@@ -193,6 +196,7 @@
                                  "yes\nn\nnot an answer\n"
                                  "no\nfoo\n")
         factory.stdout = StringIO()
+        factory.stderr = StringIO()
         # there is no output from the base factory
         self.assertEqual(True, factory.get_boolean(""))
         self.assertEqual(True, factory.get_boolean(""))
@@ -223,8 +227,10 @@
 
     def test_text_factory_prompt(self):
         # see <https://launchpad.net/bugs/365891>
-        factory = TextUIFactory(None, StringIO(), StringIO())
+        factory = TextUIFactory(None, StringIO(), StringIO(), StringIO())
         factory.prompt('foo %2e')
+        self.assertEqual('', factory.stdout.getvalue())
+        self.assertEqual('foo %2e', factory.stderr.getvalue())
 
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
@@ -263,37 +269,41 @@
         factory = SilentUIFactory()
         factory.stdin = StringIO("someuser\n\n")
         factory.stdout = StringIO()
-        self.assertEquals(None, 
+        factory.stderr = StringIO()
+        self.assertEquals(None,
             factory.get_username(u'Hello\u1234 %(host)s', host=u'some\u1234'))
         self.assertEquals("", factory.stdout.getvalue())
+        self.assertEquals("", factory.stderr.getvalue())
         self.assertEquals("someuser\n\n", factory.stdin.getvalue())
 
     def test_text_ui_getusername(self):
         factory = TextUIFactory(None, None, None)
         factory.stdin = StringIO("someuser\n\n")
         factory.stdout = StringIO()
+        factory.stderr = StringIO()
         factory.stdout.encoding = "utf8"
         # there is no output from the base factory
-        self.assertEqual("someuser", 
-            factory.get_username('Hello %(host)s', host='some'))
-        self.assertEquals("Hello some: ", factory.stdout.getvalue())
+        self.assertEqual("someuser",
+                         factory.get_username('Hello %(host)s', host='some'))
+        self.assertEquals("Hello some: ", factory.stderr.getvalue())
+        self.assertEquals('', factory.stdout.getvalue())
         self.assertEqual("", factory.get_username("Gebruiker"))
         # stdin should be empty
         self.assertEqual('', factory.stdin.readline())
 
     def test_text_ui_getusername_utf8(self):
         ui = TestUIFactory(stdin=u'someuser\u1234'.encode('utf8'),
-                           stdout=StringIOWrapper())
-        ui.stdin.encoding = "utf8"
-        ui.stdout.encoding = ui.stdin.encoding
+                           stdout=StringIOWrapper(), stderr=StringIOWrapper())
+        ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = "utf8"
         pb = ui.nested_progress_bar()
         try:
             # there is no output from the base factory
-            username = self.apply_redirected(ui.stdin, ui.stdout, ui.stdout,
+            username = self.apply_redirected(ui.stdin, ui.stdout, ui.stderr,
                 ui.get_username, u'Hello\u1234 %(host)s', host=u'some\u1234')
             self.assertEquals(u"someuser\u1234", username.decode('utf8'))
-            self.assertEquals(u"Hello\u1234 some\u1234: ", 
-                ui.stdout.getvalue().decode("utf8"))
+            self.assertEquals(u"Hello\u1234 some\u1234: ",
+                              ui.stderr.getvalue().decode("utf8"))
+            self.assertEquals('', ui.stdout.getvalue())
         finally:
             pb.finished()
 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2009-04-24 15:49:33 +0000
+++ bzrlib/ui/__init__.py	2009-05-15 10:14:51 +0000
@@ -227,7 +227,7 @@
             prompt = prompt % kwargs
         prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
         self.clear_term()
-        self.stdout.write(prompt)
+        self.stderr.write(prompt)
 
     def note(self, msg):
         """Write an already-formatted message."""

